require 'html-formatting'

class Renderer
  include HtmlFormatting

  def ping_page(app)
    page("Server Alive")
  end

  def beginning_page(app)
    page('Welcome to the Case Management System',
         p({:align => 'center'},
           "Authorized Users", b("Only")),
         command_form('login',
                      labeled_textfield('login'),
                      labeled_textfield('password'),
                      submit('Login')))
  end


  def main_page(app)
    page("Cases Available to #{app.user.capitalize}",
         p({ :align => 'center'}, "No cases are available."),
         command_form('new_case',
                      submit('Add a New Case')))
  end


  def gather_case_input_page(app)
    page("Add a Case",
         command_form('record_case',
                      labeled_textfield('client'),
                      labeled_textfield('clinic_id'),
                      submit('Add')))
  end



  def visit_list(visits)
    unless visits.empty?
      p("Visits: ", br,
        ul(visits.collect { | visit |
             li("#{visit.diagnosis} (#{visit.charges})")
           }))
    end
  end

  def audit_list(audits)
    unless audits.empty?
      p("Audits: ", br,
        ul(audits.collect { | audit |
             li("#{audit.auditor} found #{audit.variance} variance.")
           }))
    end
  end



  def case_display_page(app)
    case_record = app.current_record
    page("Case #{case_record.clinic_id}",
         p("Owner: #{case_record.client}"),
         visit_list(case_record.visits),
         audit_list(case_record.audits),
         p(command_form('add_visit',
                        submit('Add a Visit'))),
         p(command_form('add_audit',
                        submit('Add an Audit Record'))))
  end


  def gather_visit_input_page(app)
    case_record = app.current_record
    page("Add a visit to case #{case_record.clinic_id}.",
         command_form('record_visit',
                      labeled_textfield('diagnosis'),
                      labeled_textfield('charges'),
                      submit('Add')))
  end

  def gather_audit_input_page(app)
    case_record = app.current_record
    page("Add an audit to case #{case_record.clinic_id}.",
         command_form('record_audit',
                      labeled_textfield('auditor'),
                      labeled_textfield('variance'),
                      submit('Add')))
  end
end
