require 'webrick' 
require 'xmlrpc/server'
require 'paths'
require 'pp'
require 'app'
require 'renderer'

include WEBrick

class AppWebInterface

  def initialize(app)
    @server = HTTPServer.new(:Port => 8080)
    @app = app
    @renderer = Renderer.new
    @dispatched = []
    @current_page_name = :no_page_yet
    install_UI_servlets
    install_testing_servlets
    install_utility_servlets
    trap("INT"){ @server.shutdown } 
    @server.start
  end

  def install_generic_proc(mount_point, &block)
    @server.mount_proc(mount_point,
                       proc { | request, response |
                         response.body = block.call(request, @app)
                         raise HTTPStatus::OK
                       })
  end

  # TODO: check that required args are supplied.
  def install_command(command, *required_args)
    install_generic_proc("/#{command}") { | request, app |
      @current_page_name = dispatch(command, request.query)
      render(@current_page_name)
    }
  end

  def dispatch(command, args)
    @dispatched << [command, args]
    @app.send(command, args)
  end

  def render(page_name)
    @current_xhtml = @renderer.send(page_name, @app)
  end



  def install_UI_servlets
    install_generic_proc('/') { | request, app |
      render(:beginning_page)
    }
    install_command(:login, 'login', 'password')
    install_command(:new_case)
    install_command(:record_case, 'client', 'clinic_id')
    install_command(:add_visit)
    install_command(:add_audit)
    install_command(:record_visit, 'diagnosis', 'charges')
    install_command(:record_audit, 'auditor', 'variance')
  end

  def install_utility_servlets
    install_generic_proc('/ping') { | request, app |
      render(:ping_page)
    }

    install_generic_proc('/refresh') { | request, app |
      render(@current_page_name)
    }
  end


  def install_testing_servlets
    dumper_servlet = XMLRPC::WEBrickServlet.new
    @server.mount("/side-channel", dumper_servlet)

    dumper_servlet.add_handler("snapshot") {
      retval = '' 
      PP.pp(@dispatched, retval)
      retval
    }

    dumper_servlet.add_handler("replay") { | command_string |
      command_descriptions = eval(command_string)
      command_descriptions.each { | one |
        @current_page_name = dispatch(*one)
      }
      'return value irrelevant but it cannot be nil'
    }

  end


end

if __FILE__ == $0
  AppWebInterface.new(App.new)
end
